@@ -20,6 +20,12 @@
 #include "qlist.h"
 #include "qtimezone.h"
 #include "private/qlocale_p.h"
+#include "private/qdatetime_p.h"
+
+#if QT_CONFIG(timezone_tzdb)
+#include <chrono>
+#endif
+#include <optional>
 
 #if QT_CONFIG(icu)
 #include <unicode/ucal.h>
@@ -37,27 +43,45 @@ Q_FORWARD_DECLARE_OBJC_CLASS(NSTimeZone);
 #include <QJniObject>
 #endif
 
+QT_REQUIRE_CONFIG(timezone);
 QT_BEGIN_NAMESPACE
 
 class Q_AUTOTEST_EXPORT QTimeZonePrivate : public QSharedData
 {
+    // Nothing should be copy-assigning instances of either this or its derived
+    // classes (only clone() should copy, using the copy-constructor):
+    bool operator=(const QTimeZonePrivate &) const = delete;
+protected:
+    QTimeZonePrivate(const QTimeZonePrivate &other) = default;
 public:
-    //Version of QTimeZone::OffsetData struct using msecs for efficiency
+    // Version of QTimeZone::OffsetData struct using msecs for efficiency
     struct Data {
         QString abbreviation;
         qint64 atMSecsSinceEpoch;
         int offsetFromUtc;
         int standardTimeOffset;
         int daylightTimeOffset;
+        Data()
+            : atMSecsSinceEpoch(QTimeZonePrivate::invalidMSecs()),
+              offsetFromUtc(QTimeZonePrivate::invalidSeconds()),
+              standardTimeOffset(QTimeZonePrivate::invalidSeconds()),
+              daylightTimeOffset(QTimeZonePrivate::invalidSeconds())
+        {}
+        Data(const QString &name, qint64 when, int offset, int standard)
+            : abbreviation(name),
+              atMSecsSinceEpoch(when),
+              offsetFromUtc(offset),
+              standardTimeOffset(standard),
+              daylightTimeOffset(offset - standard)
+        {}
     };
     typedef QList<Data> DataList;
 
     // Create null time zone
     QTimeZonePrivate();
-    QTimeZonePrivate(const QTimeZonePrivate &other);
     virtual ~QTimeZonePrivate();
 
-    virtual QTimeZonePrivate *clone() const;
+    virtual QTimeZonePrivate *clone() const = 0;
 
     bool operator==(const QTimeZonePrivate &other) const;
     bool operator!=(const QTimeZonePrivate &other) const;
@@ -84,7 +108,17 @@ public:
     virtual bool isDaylightTime(qint64 atMSecsSinceEpoch) const;
 
     virtual Data data(qint64 forMSecsSinceEpoch) const;
-    Data dataForLocalTime(qint64 forLocalMSecs, int hint) const;
+    virtual Data data(QTimeZone::TimeType timeType) const;
+    virtual bool isDataLocale(const QLocale &locale) const;
+    static bool isAnglicLocale(const QLocale &locale)
+    {
+        // Sufficiently like the C locale for displayName()-related purposes:
+        const QLocale::Language lang = locale.language();
+        return lang == QLocale::C
+            || (lang == QLocale::English && locale.script() == QLocale::LatinScript);
+    }
+    QDateTimePrivate::ZoneState stateAtZoneTime(qint64 forLocalMSecs,
+                                                QDateTimePrivate::TransitionOptions resolve) const;
 
     virtual bool hasTransitions() const;
     virtual Data nextTransition(qint64 afterMSecsSinceEpoch) const;
@@ -94,7 +128,7 @@ public:
     virtual QByteArray systemTimeZoneId() const;
 
     virtual bool isTimeZoneIdAvailable(const QByteArray &ianaId) const;
-    virtual QList<QByteArray> availableTimeZoneIds() const;
+    virtual QList<QByteArray> availableTimeZoneIds() const = 0;
     virtual QList<QByteArray> availableTimeZoneIds(QLocale::Territory territory) const;
     virtual QList<QByteArray> availableTimeZoneIds(int utcOffset) const;
 
@@ -107,22 +141,30 @@ public:
     { return (std::numeric_limits<qint64>::min)() + 1; }
     [[nodiscard]] static constexpr qint64 invalidMSecs()
     { return (std::numeric_limits<qint64>::min)(); }
-    [[nodiscard]] static constexpr qint64 invalidSeconds()
+    [[nodiscard]] static constexpr int invalidSeconds()
     { return (std::numeric_limits<int>::min)(); }
-    static Data invalidData();
     static QTimeZone::OffsetData invalidOffsetData();
     static QTimeZone::OffsetData toOffsetData(const Data &data);
     static bool isValidId(const QByteArray &ianaId);
     static QString isoOffsetFormat(int offsetFromUtc,
                                    QTimeZone::NameType mode = QTimeZone::OffsetName);
 
+    static QByteArray aliasToIana(QByteArrayView alias);
     static QByteArray ianaIdToWindowsId(const QByteArray &ianaId);
     static QByteArray windowsIdToDefaultIanaId(const QByteArray &windowsId);
     static QByteArray windowsIdToDefaultIanaId(const QByteArray &windowsId,
-                                                QLocale::Territory territory);
+                                               QLocale::Territory territory);
     static QList<QByteArray> windowsIdToIanaIds(const QByteArray &windowsId);
     static QList<QByteArray> windowsIdToIanaIds(const QByteArray &windowsId,
-                                                 QLocale::Territory territory);
+                                                QLocale::Territory territory);
+    struct NamePrefixMatch
+    {
+        QByteArray ianaId;
+        qsizetype nameLength = 0;
+        QTimeZone::TimeType timeType = QTimeZone::GenericTime;
+    };
+    static NamePrefixMatch findLongNamePrefix(QStringView text, const QLocale &locale,
+                                              std::optional<qint64> atEpochMillis = std::nullopt);
 
     // returns "UTC" QString and QByteArray
     [[nodiscard]] static inline QString utcQString()
@@ -135,7 +177,28 @@ public:
         return QByteArrayLiteral("UTC");
     }
 
+
+#ifdef QT_BUILD_INTERNAL // For the benefit of a test
+    [[nodiscard]] static inline const QTimeZonePrivate *extractPrivate(const QTimeZone &zone)
+    {
+        return zone.d.operator->();
+    }
+#endif
+
 protected:
+    // Zones CLDR data says match a condition.
+    // Use to filter what the backend has available.
+    QList<QByteArrayView> matchingTimeZoneIds(QLocale::Territory territory) const;
+    QList<QByteArrayView> matchingTimeZoneIds(int utcOffset) const;
+
+#if QT_CONFIG(timezone_locale)
+    // Defined in qtimezonelocale.cpp
+    QString localeName(qint64 atMSecsSinceEpoch, int offsetFromUtc,
+                       QTimeZone::TimeType timeType,
+                       QTimeZone::NameType nameType,
+                       const QLocale &locale) const;
+#endif // L10n helpers.
+
     QByteArray m_id;
 };
 Q_DECLARE_TYPEINFO(QTimeZonePrivate::Data, Q_RELOCATABLE_TYPE);
@@ -144,6 +207,8 @@ template<> QTimeZonePrivate *QSharedDataPointer<QTimeZonePrivate>::clone();
 
 class Q_AUTOTEST_EXPORT QUtcTimeZonePrivate final : public QTimeZonePrivate
 {
+    bool operator=(const QUtcTimeZonePrivate &) const = delete;
+    QUtcTimeZonePrivate(const QUtcTimeZonePrivate &other);
 public:
     // Create default UTC time zone
     QUtcTimeZonePrivate();
@@ -155,19 +220,23 @@ public:
     QUtcTimeZonePrivate(const QByteArray &zoneId, int offsetSeconds, const QString &name,
                         const QString &abbreviation, QLocale::Territory territory,
                         const QString &comment);
-    QUtcTimeZonePrivate(const QUtcTimeZonePrivate &other);
     virtual ~QUtcTimeZonePrivate();
 
     // Fall-back for UTC[+-]\d+(:\d+){,2} IDs.
-    static qint64 offsetFromUtcString(const QByteArray &id);
+    static qint64 offsetFromUtcString(QByteArrayView id);
 
     QUtcTimeZonePrivate *clone() const override;
 
     Data data(qint64 forMSecsSinceEpoch) const override;
+    Data data(QTimeZone::TimeType timeType) const override;
+    bool isDataLocale(const QLocale &locale) const override;
 
     QLocale::Territory territory() const override;
     QString comment() const override;
 
+    QString displayName(qint64 atMSecsSinceEpoch,
+                        QTimeZone::NameType nameType,
+                        const QLocale &locale) const override;
     QString displayName(QTimeZone::TimeType timeType,
                         QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
@@ -186,7 +255,6 @@ public:
     void serialize(QDataStream &ds) const override;
 
 private:
-    void init(const QByteArray &zoneId);
     void init(const QByteArray &zoneId, int offsetSeconds, const QString &name,
               const QString &abbreviation, QLocale::Territory territory,
               const QString &comment);
@@ -198,18 +266,56 @@ private:
     int m_offsetFromUtc;
 };
 
-#if QT_CONFIG(icu)
-class Q_AUTOTEST_EXPORT QIcuTimeZonePrivate final : public QTimeZonePrivate
+// Platform backend cascade: match newBackendTimeZone() in qtimezone.cpp
+#if QT_CONFIG(timezone_tzdb)
+class QChronoTimeZonePrivate final : public QTimeZonePrivate
 {
+    bool operator=(const QChronoTimeZonePrivate &) const = delete;
+    QChronoTimeZonePrivate(const QChronoTimeZonePrivate &) = default;
+public:
+    QChronoTimeZonePrivate();
+    QChronoTimeZonePrivate(QByteArrayView id);
+    ~QChronoTimeZonePrivate() override;
+    QChronoTimeZonePrivate *clone() const override;
+
+    QByteArray systemTimeZoneId() const override;
+
+    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
+    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
+    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
+    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
+
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
+    QList<QByteArray> availableTimeZoneIds() const override;
+    QList<QByteArray> availableTimeZoneIds(int utcOffset) const override;
+
+    bool hasDaylightTime() const override;
+    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
+
+    Data data(qint64 forMSecsSinceEpoch) const override;
+
+    bool hasTransitions() const override;
+    Data nextTransition(qint64 afterMSecsSinceEpoch) const override;
+    Data previousTransition(qint64 beforeMSecsSinceEpoch) const override;
+
+private:
+    const std::chrono::time_zone *const m_timeZone;
+};
+#elif defined(Q_OS_DARWIN)
+class Q_AUTOTEST_EXPORT QMacTimeZonePrivate final : public QTimeZonePrivate
+{
+    bool operator=(const QMacTimeZonePrivate &) const = delete;
+    QMacTimeZonePrivate(const QMacTimeZonePrivate &other);
 public:
     // Create default time zone
-    QIcuTimeZonePrivate();
+    QMacTimeZonePrivate();
     // Create named time zone
-    QIcuTimeZonePrivate(const QByteArray &ianaId);
-    QIcuTimeZonePrivate(const QIcuTimeZonePrivate &other);
-    ~QIcuTimeZonePrivate();
+    QMacTimeZonePrivate(const QByteArray &ianaId);
+    ~QMacTimeZonePrivate();
 
-    QIcuTimeZonePrivate *clone() const override;
+    QMacTimeZonePrivate *clone() const override;
+
+    QString comment() const override;
 
     using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
@@ -223,143 +329,7 @@ public:
     bool hasDaylightTime() const override;
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
-    Data data(qint64 forMSecsSinceEpoch) const override;
-
-    bool hasTransitions() const override;
-    Data nextTransition(qint64 afterMSecsSinceEpoch) const override;
-    Data previousTransition(qint64 beforeMSecsSinceEpoch) const override;
-
-    QByteArray systemTimeZoneId() const override;
-
-    QList<QByteArray> availableTimeZoneIds() const override;
-    QList<QByteArray> availableTimeZoneIds(QLocale::Territory territory) const override;
-    QList<QByteArray> availableTimeZoneIds(int offsetFromUtc) const override;
-
-private:
-    void init(const QByteArray &ianaId);
-
-    UCalendar *m_ucal;
-};
-#endif
-
-#if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN) && !defined(Q_OS_ANDROID)
-struct QTzTransitionTime
-{
-    qint64 atMSecsSinceEpoch;
-    quint8 ruleIndex;
-};
-Q_DECLARE_TYPEINFO(QTzTransitionTime, Q_PRIMITIVE_TYPE);
-struct QTzTransitionRule
-{
-    int stdOffset;
-    int dstOffset;
-    quint8 abbreviationIndex;
-};
-Q_DECLARE_TYPEINFO(QTzTransitionRule, Q_PRIMITIVE_TYPE);
-constexpr inline bool operator==(const QTzTransitionRule &lhs, const QTzTransitionRule &rhs) noexcept
-{ return lhs.stdOffset == rhs.stdOffset && lhs.dstOffset == rhs.dstOffset && lhs.abbreviationIndex == rhs.abbreviationIndex; }
-constexpr inline bool operator!=(const QTzTransitionRule &lhs, const QTzTransitionRule &rhs) noexcept
-{ return !operator==(lhs, rhs); }
-
-// These are stored separately from QTzTimeZonePrivate so that they can be
-// cached, avoiding the need to re-parse them from disk constantly.
-struct QTzTimeZoneCacheEntry
-{
-    QList<QTzTransitionTime> m_tranTimes;
-    QList<QTzTransitionRule> m_tranRules;
-    QList<QByteArray> m_abbreviations;
-    QByteArray m_posixRule;
-    QTzTransitionRule m_preZoneRule;
-    bool m_hasDst;
-};
-
-class Q_AUTOTEST_EXPORT QTzTimeZonePrivate final : public QTimeZonePrivate
-{
-    QTzTimeZonePrivate(const QTzTimeZonePrivate &) = default;
-public:
-    // Create default time zone
-    QTzTimeZonePrivate();
-    // Create named time zone
-    QTzTimeZonePrivate(const QByteArray &ianaId);
-    ~QTzTimeZonePrivate();
-
-    QTzTimeZonePrivate *clone() const override;
-
-    QLocale::Territory territory() const override;
-    QString comment() const override;
-
-    QString displayName(qint64 atMSecsSinceEpoch,
-                        QTimeZone::NameType nameType,
-                        const QLocale &locale) const override;
-    QString displayName(QTimeZone::TimeType timeType,
-                        QTimeZone::NameType nameType,
-                        const QLocale &locale) const override;
-    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
-
-    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
-    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
-    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
-
-    bool hasDaylightTime() const override;
-    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
-
-    Data data(qint64 forMSecsSinceEpoch) const override;
-
-    bool hasTransitions() const override;
-    Data nextTransition(qint64 afterMSecsSinceEpoch) const override;
-    Data previousTransition(qint64 beforeMSecsSinceEpoch) const override;
-
-    QByteArray systemTimeZoneId() const override;
-
-    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
-    QList<QByteArray> availableTimeZoneIds() const override;
-    QList<QByteArray> availableTimeZoneIds(QLocale::Territory territory) const override;
-
-private:
-    static QByteArray staticSystemTimeZoneId();
-    QList<QTimeZonePrivate::Data> getPosixTransitions(qint64 msNear) const;
-
-    Data dataForTzTransition(QTzTransitionTime tran) const;
-    Data dataFromRule(QTzTransitionRule rule, qint64 msecsSinceEpoch) const;
-#if QT_CONFIG(icu)
-# ifdef __cpp_lib_is_final
-    static_assert(std::is_final<QIcuTimeZonePrivate>::value,
-                  "if QIcuTimeZonePrivate isn't final, we may need to specialize "
-                  "QExplicitlySharedDataPointer::clone() to call QTimeZonePrivate::clone()");
-# endif
-    mutable QExplicitlySharedDataPointer<const QIcuTimeZonePrivate> m_icu;
-#endif
-    QTzTimeZoneCacheEntry cached_data;
-    const QList<QTzTransitionTime> &tranCache() const { return cached_data.m_tranTimes; }
-};
-#endif // Q_OS_UNIX
-
-#ifdef Q_OS_DARWIN
-class Q_AUTOTEST_EXPORT QMacTimeZonePrivate final : public QTimeZonePrivate
-{
-public:
-    // Create default time zone
-    QMacTimeZonePrivate();
-    // Create named time zone
-    QMacTimeZonePrivate(const QByteArray &ianaId);
-    QMacTimeZonePrivate(const QMacTimeZonePrivate &other);
-    ~QMacTimeZonePrivate();
-
-    QMacTimeZonePrivate *clone() const override;
-
-    QString comment() const override;
-
-    QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
-                        const QLocale &locale) const override;
-    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
-
-    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
-    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
-    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
-
-    bool hasDaylightTime() const override;
-    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
-
+    using QTimeZonePrivate::data;
     Data data(qint64 forMSecsSinceEpoch) const override;
 
     bool hasTransitions() const override;
@@ -377,11 +347,176 @@ private:
 
     NSTimeZone *m_nstz;
 };
-#endif // Q_OS_DARWIN
+#elif defined(Q_OS_ANDROID)
+class QAndroidTimeZonePrivate final : public QTimeZonePrivate
+{
+    bool operator=(const QAndroidTimeZonePrivate &) const = delete;
+    QAndroidTimeZonePrivate(const QAndroidTimeZonePrivate &) = default;
+public:
+    // Create default time zone
+    QAndroidTimeZonePrivate();
+    // Create named time zone
+    QAndroidTimeZonePrivate(const QByteArray &ianaId);
+    ~QAndroidTimeZonePrivate();
 
-#if defined(Q_OS_WIN) && !QT_CONFIG(icu)
+    QAndroidTimeZonePrivate *clone() const override;
+
+    using QTimeZonePrivate::displayName;
+    QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
+                        const QLocale &locale) const override;
+    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
+
+    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
+    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
+    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
+
+    bool hasDaylightTime() const override;
+    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
+
+    using QTimeZonePrivate::data;
+    Data data(qint64 forMSecsSinceEpoch) const override;
+
+    QByteArray systemTimeZoneId() const override;
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
+    QList<QByteArray> availableTimeZoneIds() const override;
+
+private:
+    void init(const QByteArray &zoneId);
+
+    QJniObject androidTimeZone;
+};
+#elif defined(Q_OS_UNIX)
+struct QTzTransitionTime
+{
+    qint64 atMSecsSinceEpoch;
+    quint8 ruleIndex;
+};
+Q_DECLARE_TYPEINFO(QTzTransitionTime, Q_PRIMITIVE_TYPE);
+struct QTzTransitionRule
+{
+    int stdOffset = 0;
+    int dstOffset = 0;
+    quint8 abbreviationIndex = 0;
+};
+Q_DECLARE_TYPEINFO(QTzTransitionRule, Q_PRIMITIVE_TYPE);
+constexpr inline bool operator==(const QTzTransitionRule &lhs, const QTzTransitionRule &rhs) noexcept
+{ return lhs.stdOffset == rhs.stdOffset && lhs.dstOffset == rhs.dstOffset && lhs.abbreviationIndex == rhs.abbreviationIndex; }
+constexpr inline bool operator!=(const QTzTransitionRule &lhs, const QTzTransitionRule &rhs) noexcept
+{ return !operator==(lhs, rhs); }
+
+// These are stored separately from QTzTimeZonePrivate so that they can be
+// cached, avoiding the need to re-parse them from disk constantly.
+struct QTzTimeZoneCacheEntry
+{
+    QList<QTzTransitionTime> m_tranTimes;
+    QList<QTzTransitionRule> m_tranRules;
+    QList<QByteArray> m_abbreviations;
+    QByteArray m_posixRule;
+    QTzTransitionRule m_preZoneRule;
+    bool m_hasDst = false;
+};
+
+class Q_AUTOTEST_EXPORT QTzTimeZonePrivate final : public QTimeZonePrivate
+{
+    bool operator=(const QTzTimeZonePrivate &) const = delete;
+    QTzTimeZonePrivate(const QTzTimeZonePrivate &) = default;
+public:
+    // Create default time zone
+    QTzTimeZonePrivate();
+    // Create named time zone
+    QTzTimeZonePrivate(const QByteArray &ianaId);
+    ~QTzTimeZonePrivate();
+
+    QTzTimeZonePrivate *clone() const override;
+
+    QLocale::Territory territory() const override;
+    QString comment() const override;
+
+    using QTimeZonePrivate::displayName;
+    QString displayName(QTimeZone::TimeType timeType,
+                        QTimeZone::NameType nameType,
+                        const QLocale &locale) const override;
+    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
+
+    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
+    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
+    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
+
+    bool hasDaylightTime() const override;
+    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
+
+    Data data(qint64 forMSecsSinceEpoch) const override;
+    Data data(QTimeZone::TimeType timeType) const override;
+    bool isDataLocale(const QLocale &locale) const override;
+
+    bool hasTransitions() const override;
+    Data nextTransition(qint64 afterMSecsSinceEpoch) const override;
+    Data previousTransition(qint64 beforeMSecsSinceEpoch) const override;
+
+    QByteArray systemTimeZoneId() const override;
+
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
+    QList<QByteArray> availableTimeZoneIds() const override;
+    QList<QByteArray> availableTimeZoneIds(QLocale::Territory territory) const override;
+
+private:
+    static QByteArray staticSystemTimeZoneId();
+    QList<QTimeZonePrivate::Data> getPosixTransitions(qint64 msNear) const;
+
+    Data dataForTzTransition(QTzTransitionTime tran) const;
+    Data dataFromRule(QTzTransitionRule rule, qint64 msecsSinceEpoch) const;
+    QTzTimeZoneCacheEntry cached_data;
+    const QList<QTzTransitionTime> &tranCache() const { return cached_data.m_tranTimes; }
+};
+#elif QT_CONFIG(icu)
+class Q_AUTOTEST_EXPORT QIcuTimeZonePrivate final : public QTimeZonePrivate
+{
+    bool operator=(const QIcuTimeZonePrivate &) const = delete;
+    QIcuTimeZonePrivate(const QIcuTimeZonePrivate &other);
+public:
+    // Create default time zone
+    QIcuTimeZonePrivate();
+    // Create named time zone
+    QIcuTimeZonePrivate(const QByteArray &ianaId);
+    ~QIcuTimeZonePrivate();
+
+    QIcuTimeZonePrivate *clone() const override;
+
+    using QTimeZonePrivate::displayName;
+    QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
+                        const QLocale &locale) const override;
+
+    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
+    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
+    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
+
+    bool hasDaylightTime() const override;
+    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
+
+    using QTimeZonePrivate::data;
+    Data data(qint64 forMSecsSinceEpoch) const override;
+
+    bool hasTransitions() const override;
+    Data nextTransition(qint64 afterMSecsSinceEpoch) const override;
+    Data previousTransition(qint64 beforeMSecsSinceEpoch) const override;
+
+    QByteArray systemTimeZoneId() const override;
+
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
+    QList<QByteArray> availableTimeZoneIds() const override;
+    QList<QByteArray> availableTimeZoneIds(QLocale::Territory territory) const override;
+    QList<QByteArray> availableTimeZoneIds(int offsetFromUtc) const override;
+
+private:
+    void init(const QByteArray &ianaId);
+
+    UCalendar *m_ucal;
+};
+#elif defined(Q_OS_WIN)
 class Q_AUTOTEST_EXPORT QWinTimeZonePrivate final : public QTimeZonePrivate
 {
+    bool operator=(const QWinTimeZonePrivate &) const = delete;
+    QWinTimeZonePrivate(const QWinTimeZonePrivate &) = default;
 public:
     struct QWinTransitionRule {
         int startYear;
@@ -395,13 +530,13 @@ public:
     QWinTimeZonePrivate();
     // Create named time zone
     QWinTimeZonePrivate(const QByteArray &ianaId);
-    QWinTimeZonePrivate(const QWinTimeZonePrivate &other);
     ~QWinTimeZonePrivate();
 
     QWinTimeZonePrivate *clone() const override;
 
     QString comment() const override;
 
+    using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
     QString abbreviation(qint64 atMSecsSinceEpoch) const override;
@@ -413,6 +548,7 @@ public:
     bool hasDaylightTime() const override;
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
+    using QTimeZonePrivate::data;
     Data data(qint64 forMSecsSinceEpoch) const override;
 
     bool hasTransitions() const override;
@@ -435,45 +571,7 @@ private:
     QString m_daylightName;
     QList<QWinTransitionRule> m_tranRules;
 };
-#endif // Q_OS_WIN && !icu
-
-#ifdef Q_OS_ANDROID
-class QAndroidTimeZonePrivate final : public QTimeZonePrivate
-{
-public:
-    // Create default time zone
-    QAndroidTimeZonePrivate();
-    // Create named time zone
-    QAndroidTimeZonePrivate(const QByteArray &ianaId);
-    QAndroidTimeZonePrivate(const QAndroidTimeZonePrivate &other);
-    ~QAndroidTimeZonePrivate();
-
-    QAndroidTimeZonePrivate *clone() const override;
-
-    QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
-                        const QLocale &locale) const override;
-    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
-
-    int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
-    int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
-    int daylightTimeOffset(qint64 atMSecsSinceEpoch) const override;
-
-    bool hasDaylightTime() const override;
-    bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
-
-    Data data(qint64 forMSecsSinceEpoch) const override;
-
-    QByteArray systemTimeZoneId() const override;
-
-    QList<QByteArray> availableTimeZoneIds() const override;
-
-private:
-    void init(const QByteArray &zoneId);
-
-    QJniObject androidTimeZone;
-
-};
-#endif // Q_OS_ANDROID
+#endif // C++20, Darwin, Android, Unix, ICU, Win.
 
 QT_END_NAMESPACE
 
