@@ -4,8 +4,6 @@
 
 #define QT_FEATURE_cxx17_filesystem 1
 
-#define QT_FEATURE_eventfd 1
-
 #define QT_FEATURE_glib 1
 
 #define QT_FEATURE_inotify 1
@@ -76,6 +74,8 @@
 
 #define QT_FEATURE_timezone 1
 
+#define QT_FEATURE_timezone_tzdb -1
+
 #define QT_FEATURE_commandlineparser 1
 
 #define QT_FEATURE_cborstreamreader 1
