@@ -289,7 +289,7 @@ QList<int> QMainWindowLayoutSeparatorHelper<Layout>::findSeparator(const QPoint
     Layout *layout = const_cast<Layout*>(this->layout());
 #if QT_CONFIG(toolbar)
     QToolBarAreaLayout *toolBarAreaLayout = layout->toolBarAreaLayout();
-    if (!toolBarAreaLayout->isEmpty()) {
+    if (toolBarAreaLayout && !toolBarAreaLayout->isEmpty()) {
         // We might have a toolbar that is currently expanded, covering
         // parts of the dock area, in which case we don't want the dock
         // area layout to treat mouse events for the expanded toolbar as
@@ -363,7 +363,7 @@ public:
     void restore();
     void apply();
     void childEvent(QChildEvent *event) override;
-    void reparent(QDockWidget *dockWidget);
+    void reparentToMainWindow(QDockWidget *dockWidget);
     void destroyIfSingleItemLeft();
     QList<QDockWidget *> dockWidgets() const { return findChildren<QDockWidget *>(); }
 
@@ -537,7 +537,7 @@ public:
     void splitDockWidget(QDockWidget *after,
                          QDockWidget *dockwidget,
                          Qt::Orientation orientation);
-    Qt::DockWidgetArea dockWidgetArea(QWidget* widget) const;
+    Qt::DockWidgetArea dockWidgetArea(const QWidget* widget) const;
     bool restoreDockWidget(QDockWidget *dockwidget);
 #if QT_CONFIG(tabbar)
     void tabifyDockWidget(QDockWidget *first, QDockWidget *second);
@@ -550,8 +550,8 @@ public:
     void setDocumentMode(bool enabled);
 
     QTabBar *getTabBar();
+    void unuseTabBar(QTabBar *bar);
     QSet<QTabBar*> usedTabBars;
-    QList<QTabBar*> unusedTabBars;
     bool verticalTabsEnabled;
 
     QWidget *getSeparatorWidget();
